package org.mercury_im.messenger.core.viewmodel.accounts;

import org.mercury_im.messenger.core.data.repository.AccountRepository;
import org.mercury_im.messenger.core.viewmodel.MercuryViewModel;
import org.mercury_im.messenger.entity.Account;
import org.mercury_im.messenger.core.util.ThreadUtils;
import org.mercury_im.messenger.core.xmpp.MercuryConnectionManager;
import org.mercury_im.messenger.core.xmpp.state.ConnectionPoolState;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.inject.Named;

import io.reactivex.Observable;
import io.reactivex.Scheduler;

public class AccountsViewModel implements MercuryViewModel {

    private static final Logger LOGGER = Logger.getLogger(AccountsViewModel.class.getName());

    private final MercuryConnectionManager connectionManager;
    private final AccountRepository accountRepository;
    private final Scheduler ioScheduler;
    private final Scheduler uiScheduler;

    public AccountsViewModel(MercuryConnectionManager connectionManager,
                             AccountRepository accountRepository,
                             @Named(ThreadUtils.SCHEDULER_IO) Scheduler ioScheduler,
                             @Named(ThreadUtils.SCHEDULER_UI) Scheduler uiScheduler) {

        this.connectionManager = connectionManager;
        this.accountRepository = accountRepository;
        this.ioScheduler = ioScheduler;
        this.uiScheduler = uiScheduler;
    }


    public Observable<ConnectionPoolState> getConnectionPool() {
        return connectionManager.observeConnectionPool();
    }

    public void onToggleAccountEnabled(Account account, boolean enabled) {
        account.setEnabled(enabled);
        addDisposable(accountRepository.upsertAccount(account)
                .subscribeOn(ioScheduler)
                .observeOn(uiScheduler)
                .subscribe(
                        success -> logAccountToggledSuccess(success, enabled),
                        error -> logAccountToggleError(account, enabled, error)
                )
        );
    }

    private void logAccountToggledSuccess(Account account, boolean enabled) {
        LOGGER.log(Level.FINER, "Account " + account.getAddress() + (enabled ? " enabled" : " disabled"));
    }

    private void logAccountToggleError(Account account, boolean enabled, Throwable error) {
        LOGGER.log(Level.SEVERE, "Account " + account.getAddress() + " could not be " + (enabled ? "enabled" : "disabled"), error);
    }
}
